/**
 * Module dependencies
 */

var _ = require('lodash')
var util = require('util')

/**
 *
 * @param  {[type]} scope [description]
 * @return {[type]}       [description]
 */
module.exports = function dataForPackageJSON (scope) {
  var sailsPkg = scope.sailsPackageJSON || {}

  // To determine the sails dep. to inject in the newly created package.json,
  // use `sails.prerelease` specified in the package.json of Sails itself.
  // If a `prerelease` version no. is not specified, just use `version`
  var sailsVersionDependency = (sailsPkg.sails && sailsPkg.sails.prerelease) || ('~' + sailsPkg.version)

  // List default dependencies used for apps with a front-end
  var dependenciesFrontAndBackend = {
    'ejs': getDependencyVersion(sailsPkg, 'ejs'),
    'gulp': '3.9.0',
    'gulp-autoprefixer': '2.3.1',
    'gulp-cache': '0.2.10',
    'gulp-changed': '1.2.1',
    'gulp-coffee': '2.3.1',
    'gulp-concat': '2.6.0',
    'gulp-if': '1.2.5',
    'gulp-imagemin': '2.3.0',
    'gulp-jade': '1.0.1',
    'gulp-jshint': '1.11.0',
    'gulp-less': '3.0.3',
    'gulp-linker': '0.1.7',
    'gulp-livereload': '3.8.0',
    'gulp-load-plugins': '1.0.0-rc.1',
    'gulp-minify-css': '1.2.0',
    'gulp-notify': '2.2.0',
    'gulp-rename': '1.2.2',
    'gulp-rimraf': '0.1.1',
    'gulp-sass': '2.0.2',
    'gulp-template-compile': '1.0.0',
    'gulp-uglify': '1.2.0',
    'gulp-util': '3.0.6',
    'include-all': getDependencyVersion(sailsPkg, 'include-all'),
    'rc': getDependencyVersion(sailsPkg, 'rc'),
    'sails': sailsVersionDependency,
    'sails-disk': getDependencyVersion(sailsPkg, 'sails-disk')
  }

  // List default dependencies used for back-end only apps (--no-front-end)
  var dependenciesBackendOnly = {
    'include-all': getDependencyVersion(sailsPkg, 'include-all'),
    'rc': getDependencyVersion(sailsPkg, 'rc'),
    'sails': sailsVersionDependency,
    'sails-disk': getDependencyVersion(sailsPkg, 'sails-disk')
  }

  // Creating default package.json file content
  var defaultPackageJSONContent = {
    name: scope.appName,
    private: true,
    version: '0.0.0',
    description: 'a Sails application',
    keywords: [],
    dependencies: (scope['front-end'] === false ? dependenciesBackendOnly : dependenciesFrontAndBackend),
    scripts: {
      debug: 'node debug app.js',
      start: 'node app.js'
    },
    main: 'app.js',
    repository: {
      type: 'git',
      url: util.format('git://github.com/%s/%s.git', scope.github.username, scope.appName)
    },
    author: scope.author || '',
    license: ''
  }

  //
  // Check for `packageJson` configuration
  //

  if (scope.packageJson && _.isObject(scope.packageJson)) {
    //
    // Adding new dependencies to package.json
    //
    _.merge(defaultPackageJSONContent, (scope.packageJson || {}))

    //
    // Remove dependencies that has false as version
    // If somebody don't need dependency it could be removed using passing to scope:
    //
    // ```
    // packageJson: {
    //    dependencies: {
    //      ejs: false
    //    }
    // }
    // ```
    //
    if (scope.packageJson.dependencies) {
      defaultPackageJSONContent.dependencies = _.omit(defaultPackageJSONContent.dependencies, function (value) {
        return value === false
      })
    }
  }

  return _.defaults(scope.appPackageJSON || {}, defaultPackageJSONContent)
}

/**
 * getDependencyVersion
 *
 * @param  {Object} packageJSON
 * @param  {String} module
 * @return {String}
 * @api private
 */

function getDependencyVersion (packageJSON, module) {
  return (
    packageJSON.dependencies && packageJSON.dependencies[module] ||
    packageJSON.devDependencies && packageJSON.devDependencies[module] ||
    packageJSON.optionalDependencies && packageJSON.optionalDependencies[module]
  )
}
